#ifndef EMV_TRANSACTION_H
#define EMV_TRANSACTION_H

#include "EmvKernel.h"
#include "QVSDC.h"


#define LOG_TP_COM 1    // Ƿӡlog



#define MAX_SDA_DATA				2048
#define MAX_CDA_DATA				1024
#define MAX_ISSUER_SCRIPT			300
#define MAX_ISSUER_SCRIPT_RESULT	256

#define MAX_RSA_EXPONENTS  3
#define MAX_RSA_MODULES    256

#define OFFLINE_APPROVED   "Y1" // ѻ׼
#define OFFLINE_DECLINE    "Z1" // ѻܾ
#define ONLINE_APPROVED    "Y2" // ׼
#define ONLINE_DECLINE     "Z2" // ܾ
#define ONLINE_ERROR_OFFLINE_APPROVED "Y3" // ѻ׼
#define ONLINE_ERROR_OFFLINE_DECLINE  "Z3" // ѻܾ

/**
 * Կ
 */
struct EMV_PUBLIC_KEY {
	EMV_UINT16 m_iExponentsLen;// ָ
	EMV_BYTE m_bExponentsValue[MAX_RSA_EXPONENTS];// ָ
	EMV_UINT16 m_iModuleLen;
	EMV_BYTE m_bModuleValue[MAX_RSA_MODULES]; // Ĥ
};

typedef struct EMV_PUBLIC_KEY * LP_EMV_PUBLIC_KEY;
typedef const struct EMV_PUBLIC_KEY * LPC_EMV_PUBLIC_KEY;

//#if !defined(EMV_AID_ITEM_D)
//#define EMV_AID_ITEM_D

// * AID 
struct EMV_AID_ITEM {
	//  596
	EMV_UINT16 m_uiAID;
	EMV_BYTE m_AID[MAX_AID];
	EMV_BYTE m_AppVersion[SIZE_APP_VERSION];
	EMV_BOOL m_PartMach;
	EMV_BYTE m_TerminalAcquireID[SIZE_TERMINAA_ACQUIRE_ID];
	EMV_BYTE m_Default_DDOL[MAX_DATA_SIZE]; //use when DDA
	EMV_UINT32 m_Default_DDOL_Len;
	EMV_BYTE m_Default_TDOL[MAX_DATA_SIZE]; //use when CDA
	EMV_UINT32 m_Default_TDOL_Len;
	
	EMV_UINT32 m_TerminalFloorLimit;
	EMV_UINT32 m_TresholdValue;
	EMV_UINT32 m_TargetPercentage;
	EMV_UINT32 m_MaxTargetPercentage;

	EMV_BYTE m_TAC_Denial[SIZE_ACTION_CODE];
	EMV_BYTE m_TAC_Online[SIZE_ACTION_CODE];
	EMV_BYTE m_TAC_Default[SIZE_ACTION_CODE];

	EMV_UINT32 m_EC_Trans_Limite; //EC

	EMV_UINT32 m_ContactlessTransLimit; //qPBOC
	EMV_UINT32 m_ContactlessFloorLimit; //qPBOC
	EMV_UINT32 m_ContactlessCVMAmount; //qPBOC
};

typedef struct EMV_AID_ITEM * LP_EMV_AID_ITEM;
typedef const struct EMV_AID_ITEM * LPC_EMV_AID_ITEM;

//#endif


/**
 * ׹ .
 */
struct EMV_TRANSACTION_DATA {
	struct EMV_AID_ITEM m_CurrentApp;// ǰaid
	EMV_UINT16 m_SW;// ״̬

	EMV_BYTE m_SDA_Data[256 * 32];// sda
	EMV_UINT16 m_SDA_DataSize;
	EMV_BOOL m_SDARecordIncorrect;// sdaָɹ
	EMV_BYTE m_PDOL_Data[MAX_CDA_DATA];// pdol
	EMV_UINT16 m_PDOL_DataSize;// sda

	EMV_BOOL m_CAPK_Recovered;// Կָʶ
	struct EMV_PUBLIC_KEY m_CAPK;// Կ
	EMV_BOOL m_IPK_Recovered;//йԿָʶ
	struct EMV_PUBLIC_KEY m_IPK;// йԿ
	EMV_BOOL m_ICCPK_Recovered;// icԿָʶ
	struct EMV_PUBLIC_KEY m_ICCPK;// ICԿ

	EMV_CHAR m_OnlineEncipherPIN[SIZE_PIN_BLOCK * 2 + 1];// PINBLOCK
	EMV_CHAR m_IssueScript[MAX_ISSUER_SCRIPT];// нű
	EMV_UINT16 m_IssueScriptSize;// нűݳ
	EMV_BYTE m_ScriptResults[MAX_ISSUER_SCRIPT_RESULT];// ű
	EMV_UINT16 m_uiScriptResults;// ű
	EMV_BOOL m_SignatureRequired;// ǩ
	EMV_BOOL m_OnlinePinRequired;// pin
	//EMV_BOOL         m_OnlineNeeded;
	EMV_ONLINE_RESULT m_OnlineResult;// 

	EMV_BOOL m_QVSDC_NeedToSaveLog;// log
	EMV_UINT32 m_QVSDC_TarnsCounter;// ׼
	EMV_BYTE m_QVSDC_TransType;// 
	EMV_UINT32 m_QVSDC_Amount;// ׽
	EMV_UINT32 m_QVSDC_AmountOther;// 
	EMV_BOOL m_QVSDC_Online_Flag;// ־
	EMV_BOOL m_QVSDC_Decline_Flag;// ܾ־
	EMV_BOOL m_QVSDC_AppNotAllow_Flag;// Ӧò

	
};

/**
 * ׹̲
 *
 */
struct EMV_TRANS_HANDLE {
	struct EMV_CALLBACK_FUNCS m_CallBacks;
	struct EMV_TERMINAL_INFO m_TerminalInfo;
	struct EMV_KERNEL_CONFIG m_KernelConfig;
	struct QVSDC_PARAMETER m_QVSDC_Param;
	struct QPBOC_PARAMETER m_QPBOC_Param;
	KERNEL_VERSION_TYPE m_KernelVersion;
	KERNEL_MODE_TYPE m_KernelMode;
	EMV_UINT8 m_KernelApplyAid; // ǽ޶
	EMV_UINT32 m_LogLevel;
	struct EMV_TRANSACTION_DATA m_TransactionData;
};

typedef struct EMV_TRANS_HANDLE * LP_EMV_TRANS_HANDLE;

#endif
