#ifndef SN_ZPOS_H
#define SN_ZPOS_H

#define SN_ZPOS_CUSTOMER_NAME_SIZE 20
#define SN_ZPOS_MODEL_SIZE 10
#define SN_ZPOS_CMD_SIZE 5
#define SN_ZPOS_SN_SIZE 64
#define SN_ZPOS_E_SIZE 50
#define SN_ZPOS_D_SIZE 2048
#define SN_ZPOS_SNKEY_SIZE 1024
#define SN_ZPOS_KCV_SIZE 10

#define SN_ZPOS_PACKAGES_SIZE 2048
#define SN_ZPOS_MINI 64
#define SN_ZPOS_TIPS_LISTS_SIZE 22

#define SN_ZPOS_PRIVATE_KEY_DEF_INDEX 4

#define SN_ZPOS_CMD_REQUEST "req"

typedef enum
{
    SN_ZPOS_STATE_OK = 0x0,
    SN_ZPOS_STATE_NET_BAD,
    SN_ZPOS_STATE_INIT_BAD,
    SN_ZPOS_STATE_PACK_BAD,
    SN_ZPOS_STATE_NET_CONNECT_BAD,
    SN_ZPOS_STATE_RESEV_BAD,
    SN_ZPOS_STATE_SEND_BAD,
    SN_ZPOS_STATE_RESEV_DATA_BAD,
    SN_ZPOS_STATE_DECRYT_BAD,
    SN_ZPOS_STATE_FAIL,
    SN_ZPOS_STATE_MEM_FAIL,
    SN_ZPOS_STATE_GEN_RSA_KEY,
    SN_ZPOS_STATE_WRITE_RSA_KEY,
    SN_ZPOS_STATE_CONV_TO_BIG,
    SN_ZPOS_STATE_GET_CUSTOMER,
    SN_ZPOS_STATE_SET_MED,
    SN_ZPOS_STATE_GET_SN,
    SN_ZPOS_STATE_UKNOWN_ERROR,
    SN_ZPOS_STATE_BUF_SMALL,
    SN_ZPOS_STATE_PAR_NULL,
    SN_ZPOS_STATE_INT_TO_BYTES,
    SN_ZPOS_STATE_RESEV_CMD,
} SN_ZPOS_STATE;

typedef enum
{
    SN_ZPOS_TYPE_PUBLIC_D = 0x44,
    SN_ZPOS_TYPE_PUBLIC_E = 0x45,
    SN_ZPOS_TYPE_CUSTOMER_NAME = 0x46,
    SN_ZPOS_TYPE_CUSTOMER_SUBNAME = 0x47,
    SN_ZPOS_TYPE_CMD = 0x48,
    SN_ZPOS_TYPE_SN = 0x49,
    SN_ZPOS_TYPE_MODEL = 0x50,
    SN_ZPOS_TYPE_SNKEY = 0x51,
    SN_ZPOS_TYPE_CHECK = 0x52,
    SN_ZPOS_TYPE_LRC = 0x127,
} SN_ZPOS_PROTOCOL_PAR;

typedef struct
{
    int privateKeyIndex;
    int snKeySize;
    int kcvSize;
    int eSize;
    int dSize;

    char customerName[SN_ZPOS_CUSTOMER_NAME_SIZE];
    char subCustomerName[SN_ZPOS_CUSTOMER_NAME_SIZE];
    char model[SN_ZPOS_MODEL_SIZE];
    char cmd[SN_ZPOS_CMD_SIZE];
    char sn[SN_ZPOS_SN_SIZE];
    char e[SN_ZPOS_E_SIZE];
    char d[SN_ZPOS_D_SIZE];
    char snKey[SN_ZPOS_SNKEY_SIZE];
    char kcv[SN_ZPOS_KCV_SIZE];
} SN_ZPOS_PROTOCOL;

typedef struct
{
    const char *tips;

    SN_ZPOS_STATE state;
} SN_ZPOS_ERROR_STR_LISTS;

const static SN_ZPOS_ERROR_STR_LISTS errorLists[SN_ZPOS_TIPS_LISTS_SIZE] = {
    {"SNԿװɹ", SN_ZPOS_STATE_OK},
    {"δ", SN_ZPOS_STATE_NET_BAD},
    {"ʼʧ", SN_ZPOS_STATE_INIT_BAD},
    {"", SN_ZPOS_STATE_PACK_BAD},
    {"ӷʧ", SN_ZPOS_STATE_NET_CONNECT_BAD},
    {"ݳʱ", SN_ZPOS_STATE_RESEV_BAD},
    {"ʧ", SN_ZPOS_STATE_SEND_BAD},
    {"ݴ", SN_ZPOS_STATE_RESEV_DATA_BAD},
    {"SNԿʧ", SN_ZPOS_STATE_DECRYT_BAD},
    {"SNԿװʧ", SN_ZPOS_STATE_FAIL},
    {"ڴ", SN_ZPOS_STATE_MEM_FAIL},
    {"RSAԿʧ", SN_ZPOS_STATE_GEN_RSA_KEY},
    {"дRSAԿʧ", SN_ZPOS_STATE_WRITE_RSA_KEY},
    {"תʧ", SN_ZPOS_STATE_CONV_TO_BIG},
    {"ȡͻʧ", SN_ZPOS_STATE_GET_CUSTOMER},
    {"󷽷ʧ", SN_ZPOS_STATE_SET_MED},
    {"ȡSNʧ", SN_ZPOS_STATE_GET_SN},
    {"δ֪", SN_ZPOS_STATE_UKNOWN_ERROR},
    {"̫С", SN_ZPOS_STATE_BUF_SMALL},
    {"Ϊ", SN_ZPOS_STATE_PAR_NULL},
    {"ַֽʧ", SN_ZPOS_STATE_INT_TO_BYTES},
    {"صЧ", SN_ZPOS_STATE_RESEV_CMD},
};

void snZposProtocolInt2Bytes(int src, char *value, int size);

void snZposProtocolBytes2Int(char *value, int size, int *src);

void snZposProtocolD2I(ST_GEN_RSA_KEY *sgrk, ST_RSA_KEY *srk);

int snZposProtocolView(void (*viewFlush)(int, char *));

SN_ZPOS_STATE snZposProtocolInit(SN_ZPOS_PROTOCOL **ctx);

void snZposProtocolFinish(SN_ZPOS_PROTOCOL **ctx);

SN_ZPOS_STATE snZposProtocolPak(SN_ZPOS_PROTOCOL *ctx, char *result, int *size);

SN_ZPOS_STATE snZposProtocolUnmarshal(SN_ZPOS_PROTOCOL *ctx, const char *value, int size);

void snZposProtocolSetCustomerName(SN_ZPOS_PROTOCOL *ctx, const char *value, int size);

void snZposProtocolSetSubCutomerName(SN_ZPOS_PROTOCOL *ctx, const char *value, int size);

void snZposProtocolSetModel(SN_ZPOS_PROTOCOL *ctx, const char *value, int size);

void snZposProtocolSetCmd(SN_ZPOS_PROTOCOL *ctx, const char *value, int size);

void snZposProtocolSetE(SN_ZPOS_PROTOCOL *ctx, const char *value, int size);

void snZposProtocolSetD(SN_ZPOS_PROTOCOL *ctx, const char *value, int size);

const char *snZposProtocolTipsLists(SN_ZPOS_STATE code);

#endif
