#include "global.h"
#include "Pos_Api.h"

#include "logger.h"

#ifdef TAG
#undef TAG
#endif
#define TAG "Pos_Api"

int  CL_Open(void)
{
    int bRetval = POS_ERR;
    if (EMV_TRUE == Open_Picc())
    {
        bRetval = POS_OK;
    }
    return bRetval;
}

void CL_Close(void)
{
    Close_Picc();
}

CL_STATUS CL_GetStatus(void)
{
    int bRetval;

    while (1)
    {
        bRetval = Detect_Picc(0x01);
        if (bRetval == EMV_TRUE)
            return CLS_OK;

        // ϵͳ
        if (!GetSysStatus())
        {
            return CLS_ERROR;
        }

        // ûȡ
        if (!KbGetKey())
        {
            if (KEY_CANCEL == KbGetKey())
            {
                OS_LedOff(BLUE);
                return CLS_ERROR;
            }
        }

        bRetval = GetLastErrno();
        //Sleep(10);
        switch (bRetval)
        {
            case -3001:
            case -3203:
            case -3019:
            case -3303:
                return CLS_NO_CARD;
            case -3004:
            case -3204:
                return CLS_MULTI_CARD;
            default:
                return CLS_ERROR;
        }
    }
}

CL_STATUS CL_GetStatusNoDelay(void)
{
    int bRetval;

    bRetval = Detect_Picc(0x01);
    if (bRetval == EMV_TRUE)
        return CLS_OK;

    // ϵͳ
    if (!GetSysStatus())
    {
        return CLS_ERROR;
    }

    bRetval = GetLastErrno();
    LOGD("---bRetval-----%d",bRetval);
    Sleep(10);
    switch (bRetval)
    {
        case -3001:
        case -3203:
        case -3019:
        case -3303:
            return CLS_NO_CARD;
        case -3004:
        case -3204:
            return CLS_MULTI_CARD;
        default:
            return CLS_ERROR;
    }
}

void CL_RemoveCard(void)
{
    Remove_Picc(0x01, getpicclogicchannel());
}

//#define IS_LOG 0

int  CL_ExchangeApdu(const unsigned char* pApdu,
                     unsigned short uiApdu, unsigned char*  pResponse, unsigned short* uiResponse,
                     unsigned short* pSW)
{
    int bRetval = POS_OK;
    int ierror = 0;

    if (uiApdu < 4)
    {
        bRetval = POS_ERR;
    }
    else
    {
        if (POS_OK == bRetval)
        {

#if IS_LOG
            //LOGI("APDU Send: %s\n", bytesToHexString1(pApdu, uiApdu));
            logger_serial_dump_buff("APDU Send:", pApdu, uiApdu);
#endif

            ierror = SendCmd_Picc(pApdu, (int)uiApdu, pResponse, pSW);
            if (0 <= ierror)
            {
                *uiResponse = ierror;
#if IS_LOG
                if (ierror)
                {
                    //LOGI("APDU Resp: %s    SW: %04X  \n", bytesToHexString1(pResponse, ierror), *pSW);
                    logger_serial_dump_buff("APDU Resp:", pResponse, ierror);
                    LOGI("SW: %04X  \n", *pSW);
                }
                else
                {
                    LOGI("APDU Resp: NULL  SW: %04X  \n", *pSW);
                }
#endif

            }
            else
            {
                bRetval = POS_ERR;
            }
        }
    }
    return bRetval;
}

void OS_GetCurrentTime(LP_OS_TIME pTime)
{
    tsSysTime time;
    int uiRetval = GetSysLocalTime(&time);

    if (uiRetval == 0)
    {
        pTime->m_Year = time.year;
        pTime->m_Month = time.mon;
        pTime->m_Day = time.day;
        pTime->m_Hour = time.hour;
        pTime->m_Minute = time.min;
        pTime->m_Second = time.sec;
    }
    else
    {
        pTime->m_Year = 2017;
        pTime->m_Month = 12;
        pTime->m_Day = 25;
        pTime->m_Hour = 13;
        pTime->m_Minute = 07;
        pTime->m_Second = 02;
    }
}

void OS_Generate_Rand(void * pvBuffer, int uiSize)
{
#if 1
    int bRetval = POS_OK;
    unsigned char * pBuff = (unsigned char *)pvBuffer;

    bRetval = PedGetRandom(uiSize, pBuff);

    if (bRetval <= 0)
    {
        memset(pvBuffer, 0, uiSize);
    }

#else
    unsigned char* pBuff = (unsigned char *)pvBuffer;
    while (uiSize > 0)
    {
        *pBuff = rand() % 256;
        pBuff++;
        uiSize--;
    }
#endif
}

void OS_Beep(char * ptext)
{
    //AudioSetVolume(3);
    AudioDtmfPlay_OEM(DTMF_Five, 3);
    Sleep(400);
    TTS_PlayText_OEM((unsigned char *)ptext, strlen((char *)ptext));
}

int OS_LedOff(OS_LED mLed)
{
    int bRetval = POS_ERR;
    /*switch (mLed)
    {
    case BLUE:
    bRetval = CLScLedTurnOff(LED_BLUE);
    break;
    case YELLOW:
    bRetval = CLScLedTurnOff(LED_YELLOW);
    break;
    case GREEN:
    bRetval = CLScLedTurnOff(LED_GREEN);
    break;
    case RED:
    bRetval = CLScLedTurnOff(LED_RED);
    break;
    }*/
    return bRetval;
}

int  OS_LedOn(OS_LED mLed)
{
    int bRetval = POS_ERR;
    /*switch (mLed)
    {
    case BLUE:
    bRetval = CLScLedTurnOn(LED_BLUE);
    break;
    case YELLOW:
    bRetval = CLScLedTurnOn(LED_YELLOW);
    break;
    case GREEN:
    bRetval = CLScLedTurnOn(LED_GREEN);
    break;
    case RED:
    bRetval = CLScLedTurnOn(LED_RED);
    break;
    }*/
    return bRetval;
}

int PrintLogToCom(const char *fmt, ...)
{
    char buf[2048];

    va_list args;
    va_start(args, fmt);
    // bufתݣiΪЧݳ
    vsprintf(buf, fmt, args);
    va_end(args);

    logger_print_serial_fmt(TAG, LOGGER_E, buf);
    return 0;
}

int FS_Open(const char *pszFileName, unsigned char mode)
{
    int ifd = -1;
    ifd = FsOpen(pszFileName,mode);
    return ifd;
}

void FS_Close(int uiFileID)
{
    if (uiFileID >= 0)
        FsClose(uiFileID);
}

int FS_Write(int uiFileID, const void *pvData, int uiSize)
{
    int uiRetval = 0;

    uiRetval = FsWrite(uiFileID, (unsigned char *) pvData, uiSize);

    return uiRetval;
}

int FS_Read(int uiFileID, const void *pvBuffer, int uiBuffsize)
{
    int uiRetval = 0;

    uiRetval = FsRead(uiFileID, (unsigned char *) pvBuffer, uiBuffsize);
    return uiRetval;
}

void FS_Seek(int uiFileID, long uiOffSet, FS_OPS Ops)
{
    if (uiFileID >= 0)
    {
        if (Ops == FS_SEEK_SET)
        {
            FsSeek(uiFileID, uiOffSet, SEEK_SET);
        }
        else if (Ops == FS_SEEK_CUR)
        {
            FsSeek(uiFileID, uiOffSet, SEEK_CUR);
        }
        else
        {
            FsSeek(uiFileID, uiOffSet, SEEK_END);
        }
    }
}

int FS_Size(const char *pszFileName)
{
    return FsSize(pszFileName);
}

int FS_Delete(const char *pszFileName)
{
    return FsRemove(pszFileName);
}


//ļжݵbuffer(ļͷʼƫ)
int ReadFileToBuffer(const char* pszFileName, unsigned char* buffer, int offset, unsigned int size)
{
    int retval = -1;
    int fileIndex = -1;
    fileIndex = FS_Open(pszFileName,O_CREATE);
    if(fileIndex < 0)
        return ERROR;
    retval = FsSeek(fileIndex, offset, SEEK_SET);
    if(retval < 0)
    {
        FS_Close(fileIndex);
        return ERROR;
    }
    retval = FS_Read(fileIndex, buffer, size);
    if(retval != size)
    {
        BUFCLR(buffer);
        FS_Close(fileIndex);
        return ERROR;
    }
    FS_Close(fileIndex);
    return retval;
}

//Ϣдļ(ļͷʼƫ)
int WriteInfo(const char* pszfilename,const void* info, int offset, unsigned int size)
{
    int retval = -1;
    int fileIndex = -1;
    fileIndex = FS_Open(pszfilename,O_CREATE);
    if(fileIndex < 0)
        return ERROR;
    else
    {
        retval = FsSeek(fileIndex, offset, SEEK_SET);
        if(retval < 0)
            return ERROR;
        else
        {
            retval = FS_Write(fileIndex, info, size);
            if(retval != size)
            {
                FS_Close(fileIndex);
                return ERROR;
            }
            else
            {
                FS_Close(fileIndex);
                return retval;
            }
        }
    }
}
