#include "MercurySystem.h"
#include "Pos_Base64.h"

static const char base64digits[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";


void base64_encode(unsigned char *out_str, const unsigned char *in_str, int in_len)
{
    int curr_out_len = 0;
    int i = 0;
    unsigned char a, b, c;
    out_str[0] = '\0';
    if (in_len > 0) {
        while (i < in_len) {
            a = in_str[i];
            b = (i + 1 >= in_len) ? 0 : in_str[i + 1];
            c = (i + 2 >= in_len) ? 0 : in_str[i + 2];
            if (i + 2 < in_len) {
                out_str[curr_out_len++] = (base64digits[(a >> 2) & 0x3F]);
                out_str[curr_out_len++] = (base64digits[((a << 4) & 0x30) + ((b >> 4) & 0xf)]);
                out_str[curr_out_len++] = (base64digits[((b << 2) & 0x3c) + ((c >> 6) & 0x3)]);
                out_str[curr_out_len++] = (base64digits[c & 0x3F]);
            }
            else if (i + 1 < in_len) {
                out_str[curr_out_len++] = (base64digits[(a >> 2) & 0x3F]);
                out_str[curr_out_len++] = (base64digits[((a << 4) & 0x30) + ((b >> 4) & 0xf)]);
                out_str[curr_out_len++] = (base64digits[((b << 2) & 0x3c) + ((c >> 6) & 0x3)]);
                out_str[curr_out_len++] = '=';
            }
            else {
                out_str[curr_out_len++] = (base64digits[(a >> 2) & 0x3F]);
                out_str[curr_out_len++] = (base64digits[((a << 4) & 0x30) + ((b >> 4) & 0xf)]);
                out_str[curr_out_len++] = '=';
                out_str[curr_out_len++] = '=';
            }
            i += 3;
        }
        out_str[curr_out_len] = '\0';
    }
}

int base64_decode(const char *base64, char *bindata)
{
    int i, j;
    char k;
    char temp[4];
    for (i = 0, j = 0; base64[i] != '\0'; i += 4)
    {
        memset(temp, 0xFF, sizeof(temp));
        for (k = 0; k < 64; k++)
        {
            if (base64digits[k] == base64[i])
                temp[0] = k;
        }
        for (k = 0; k < 64; k++)
        {
            if (base64digits[k] == base64[i + 1])
                temp[1] = k;
        }
        for (k = 0; k < 64; k++)
        {
            if (base64digits[k] == base64[i + 2])
                temp[2] = k;
        }
        for (k = 0; k < 64; k++)
        {
            if (base64digits[k] == base64[i + 3])
                temp[3] = k;
        }

        bindata[j++] = ((char)(((char)(temp[0] << 2)) & 0xFC)) |
                       ((char)((char)(temp[1] >> 4) & 0x03));
        if (base64[i + 2] == '=')
            break;

        bindata[j++] = ((char)(((char)(temp[1] << 4)) & 0xF0)) |
                       ((char)((char)(temp[2] >> 2) & 0x0F));
        if (base64[i + 3] == '=')
            break;

        bindata[j++] = ((char)(((char)(temp[2] << 6)) & 0xF0)) |
                       ((char)(temp[3] & 0x3F));
    }
    return j;
}
