#include "Pos_x509crt.h"
#include "Pos_Api.h"
#include <string.h>

#include "mbedtls\base64.h"
#include "mbedtls\platform.h"
#include "mbedtls\cipher.h"

//#define mbedtls_free       OS_Free

#define MBEDTLS_OID_SIZE(x) (sizeof(x) - 1)
#define ADD_LEN(s)      s, MBEDTLS_OID_SIZE(s)
#define CHECK(code) if( ( ret = code ) != 0 ){ return( ret ); }
#define CHECK_RANGE(min, max, val) if( val < min || val > max ){ return( ret ); }


#define MBEDTLS_ERR_OID_NOT_FOUND                         -0x002E  /**< OID is not found. */
#define MBEDTLS_ERR_OID_BUF_TOO_SMALL                     -0x000B  /**< output buffer is too small */


typedef struct
{
    mbedtls_oid_descriptor    descriptor;
    mbedtls_md_type           md_alg;
    mbedtls_pk_type           pk_alg;
} oid_sig_alg_t;

static const oid_sig_alg_t oid_sig_alg[] =
{
#if defined(MBEDTLS_MD2_C)
    {
        { ADD_LEN( MBEDTLS_OID_PKCS1_MD2 ),        "md2WithRSAEncryption",     "RSA with MD2" },
        MBEDTLS_MD_MD2,      MBEDTLS_PK_RSA,
    },
#endif /* MBEDTLS_MD2_C */
#if defined(MBEDTLS_MD4_C)
    {
        { ADD_LEN( MBEDTLS_OID_PKCS1_MD4 ),        "md4WithRSAEncryption",     "RSA with MD4" },
        MBEDTLS_MD_MD4,      MBEDTLS_PK_RSA,
    },
#endif /* MBEDTLS_MD4_C */
#if defined(MBEDTLS_MD5_C)
    {
        { ADD_LEN( MBEDTLS_OID_PKCS1_MD5 ),        "md5WithRSAEncryption",     "RSA with MD5" },
        MBEDTLS_MD_MD5,      MBEDTLS_PK_RSA,
    },
#endif /* MBEDTLS_MD5_C */
#if defined(MBEDTLS_SHA1_C)
    /*{
        { ADD_LEN( MBEDTLS_OID_PKCS1_SHA1 ),       "sha-1WithRSAEncryption",   "RSA with SHA1" },
        MBEDTLS_MD_SHA1,     MBEDTLS_PK_RSA,
    },*/
#endif /* MBEDTLS_SHA1_C */
#if defined(MBEDTLS_SHA256_C)
    {
        { ADD_LEN( MBEDTLS_OID_PKCS1_SHA224 ),     "sha224WithRSAEncryption",  "RSA with SHA-224" },
        MBEDTLS_MD_SHA224,   MBEDTLS_PK_RSA,
    },
    {
        { ADD_LEN( MBEDTLS_OID_PKCS1_SHA256 ),     "sha256WithRSAEncryption",  "RSA with SHA-256" },
        MBEDTLS_MD_SHA256,   MBEDTLS_PK_RSA,
    },
#endif /* MBEDTLS_SHA256_C */
#if defined(MBEDTLS_SHA512_C)
    {
        { ADD_LEN( MBEDTLS_OID_PKCS1_SHA384 ),     "sha384WithRSAEncryption",  "RSA with SHA-384" },
        MBEDTLS_MD_SHA384,   MBEDTLS_PK_RSA,
    },
    {
        { ADD_LEN( MBEDTLS_OID_PKCS1_SHA512 ),     "sha512WithRSAEncryption",  "RSA with SHA-512" },
        MBEDTLS_MD_SHA512,   MBEDTLS_PK_RSA,
    },
#endif /* MBEDTLS_SHA512_C */
#if defined(MBEDTLS_SHA1_C)
    /*   {
           { ADD_LEN( MBEDTLS_OID_RSA_SHA_OBS ),      "sha-1WithRSAEncryption",   "RSA with SHA1" },
           MBEDTLS_MD_SHA1,     MBEDTLS_PK_RSA,
       },*/
#endif /* MBEDTLS_SHA1_C */
#if defined(MBEDTLS_ECDSA_C)
#if defined(MBEDTLS_SHA1_C)
    {
        { ADD_LEN( MBEDTLS_OID_ECDSA_SHA1 ),       "ecdsa-with-SHA1",      "ECDSA with SHA1" },
        MBEDTLS_MD_SHA1,     MBEDTLS_PK_ECDSA,
    },
#endif /* MBEDTLS_SHA1_C */
#if defined(MBEDTLS_SHA256_C)
    {
        { ADD_LEN( MBEDTLS_OID_ECDSA_SHA224 ),     "ecdsa-with-SHA224",    "ECDSA with SHA224" },
        MBEDTLS_MD_SHA224,   MBEDTLS_PK_ECDSA,
    },
    {
        { ADD_LEN( MBEDTLS_OID_ECDSA_SHA256 ),     "ecdsa-with-SHA256",    "ECDSA with SHA256" },
        MBEDTLS_MD_SHA256,   MBEDTLS_PK_ECDSA,
    },
#endif /* MBEDTLS_SHA256_C */
#if defined(MBEDTLS_SHA512_C)
    {
        { ADD_LEN( MBEDTLS_OID_ECDSA_SHA384 ),     "ecdsa-with-SHA384",    "ECDSA with SHA384" },
        MBEDTLS_MD_SHA384,   MBEDTLS_PK_ECDSA,
    },
    {
        { ADD_LEN( MBEDTLS_OID_ECDSA_SHA512 ),     "ecdsa-with-SHA512",    "ECDSA with SHA512" },
        MBEDTLS_MD_SHA512,   MBEDTLS_PK_ECDSA,
    },
#endif /* MBEDTLS_SHA512_C */
#endif /* MBEDTLS_ECDSA_C */
#if defined(MBEDTLS_RSA_C)
    /*  {
    { ADD_LEN( MBEDTLS_OID_RSASSA_PSS ),        "RSASSA-PSS",           "RSASSA-PSS" },
    MBEDTLS_MD_NONE,     MBEDTLS_PK_RSASSA_PSS,
    },*/
    {
        { ADD_LEN(MBEDTLS_OID_PKCS1_SM2),  "SM2", "SM2" },
        MBEDTLS_MD_SM2, MBEDTLS_PK_SM2,
    },
#endif /* MBEDTLS_RSA_C */
    {
        { NULL, 0, NULL, NULL },
        MBEDTLS_MD_NONE, MBEDTLS_PK_NONE,
    },
};

typedef struct
{
    mbedtls_oid_descriptor    descriptor;
    mbedtls_pk_type           pk_alg;
} oid_pk_alg_t;

static const oid_pk_alg_t oid_pk_alg[] =
{
    {
        { ADD_LEN(MBEDTLS_OID_PKCS1_RSA), "rsaEncryption", "RSA" },
        MBEDTLS_PK_RSA,
    },
    {
        { ADD_LEN(MBEDTLS_OID_EC_ALG_UNRESTRICTED), "id-ecPublicKey", "Generic EC key" },
        MBEDTLS_PK_ECKEY,
    },
    {
        { ADD_LEN(MBEDTLS_OID_EC_ALG_ECDH), "id-ecDH", "EC key for ECDH" },
        MBEDTLS_PK_ECKEY_DH,
    },
    {
        { NULL, 0, NULL, NULL },
        MBEDTLS_PK_NONE,
    },
};

#define FN_OID_GET_ATTR2(FN_NAME, TYPE_T, TYPE_NAME, ATTR1_TYPE, ATTR1,     \
                         ATTR2_TYPE, ATTR2)                                 \
static int FN_NAME( const mbedtls_asn1_buf *oid, ATTR1_TYPE * ATTR1, ATTR2_TYPE * ATTR2 )  \
{                                                                           \
    const TYPE_T *data = oid_ ## TYPE_NAME ## _from_asn1( oid );            \
    if( data == NULL ) return( MBEDTLS_ERR_OID_NOT_FOUND );                \
    *ATTR1 = data->ATTR1;                                                   \
    *ATTR2 = data->ATTR2;                                                   \
    return( 0 );                                                            \
}
#define FN_OID_TYPED_FROM_ASN1( TYPE_T, NAME, LIST )                        \
static const TYPE_T * oid_ ## NAME ## _from_asn1( const mbedtls_asn1_buf *oid )     \
{                                                                           \
    const TYPE_T *p = LIST;                                                 \
    const mbedtls_oid_descriptor *cur = (const mbedtls_oid_descriptor *) p;             \
    if( p == NULL || oid == NULL ) return( NULL );                          \
    while( cur->asn1 != NULL ) {                                            \
        if( cur->asn1_len == oid->len &&                                    \
            memcmp( cur->asn1, oid->p, oid->len ) == 0 ) {                  \
            return( p );                                                    \
        }                                                                   \
        p++;                                                                \
        cur = (const mbedtls_oid_descriptor *) p;                                 \
    }                                                                       \
    return( NULL );                                                         \
}

#define FN_OID_GET_DESCRIPTOR_ATTR1(FN_NAME, TYPE_T, TYPE_NAME, ATTR1_TYPE, ATTR1) \
static int FN_NAME( const mbedtls_asn1_buf *oid, ATTR1_TYPE * ATTR1 )                  \
{                                                                       \
    const TYPE_T *data = oid_ ## TYPE_NAME ## _from_asn1( oid );        \
    if( data == NULL ) return( MBEDTLS_ERR_OID_NOT_FOUND );            \
    *ATTR1 = data->descriptor.ATTR1;                                    \
    return( 0 );                                                        \
}

#define FN_OID_GET_OID_BY_ATTR2(FN_NAME, TYPE_T, LIST, ATTR1_TYPE, ATTR1,   \
                                ATTR2_TYPE, ATTR2)                          \
static int FN_NAME( ATTR1_TYPE ATTR1, ATTR2_TYPE ATTR2, const char **oid ,         \
             size_t *olen )                                                 \
{                                                                           \
    const TYPE_T *cur = LIST;                                               \
    while( cur->descriptor.asn1 != NULL ) {                                 \
        if( cur->ATTR1 == ATTR1 && cur->ATTR2 == ATTR2 ) {                  \
            *oid = cur->descriptor.asn1;                                    \
            *olen = cur->descriptor.asn1_len;                               \
            return( 0 );                                                    \
        }                                                                   \
        cur++;                                                              \
    }                                                                       \
    return( MBEDTLS_ERR_OID_NOT_FOUND );                                   \
}

#define FN_OID_GET_ATTR1(FN_NAME, TYPE_T, TYPE_NAME, ATTR1_TYPE, ATTR1) \
static int FN_NAME(const mbedtls_asn1_buf *oid, ATTR1_TYPE * ATTR1)                  \
{                                                                       \
    const TYPE_T *data = oid_ ## TYPE_NAME ## _from_asn1( oid );        \
    if( data == NULL ) return( MBEDTLS_ERR_OID_NOT_FOUND );            \
    *ATTR1 = data->ATTR1;                                               \
    return( 0 );                                                        \
}

#define FN_OID_GET_OID_BY_ATTR1(FN_NAME, TYPE_T, LIST, ATTR1_TYPE, ATTR1)   \
static int FN_NAME(ATTR1_TYPE ATTR1, const char **oid, size_t *olen)             \
{                                                                           \
    const TYPE_T *cur = LIST;                                               \
    while( cur->descriptor.asn1 != NULL ) {                                 \
        if( cur->ATTR1 == ATTR1 ) {                                         \
            *oid = cur->descriptor.asn1;                                    \
            *olen = cur->descriptor.asn1_len;                               \
            return( 0 );                                                    \
        }                                                                   \
        cur++;                                                              \
    }                                                                       \
    return( MBEDTLS_ERR_OID_NOT_FOUND );                                   \
}


FN_OID_TYPED_FROM_ASN1(oid_sig_alg_t, sig_alg, oid_sig_alg)
//FN_OID_GET_DESCRIPTOR_ATTR1(mbedtls_oid_get_sig_alg_desc, oid_sig_alg_t, sig_alg, const char *, description)
FN_OID_GET_ATTR2(mbedtls_oid_get_sig_alg, oid_sig_alg_t, sig_alg, mbedtls_md_type, md_alg, mbedtls_pk_type, pk_alg)
//FN_OID_GET_OID_BY_ATTR2(mbedtls_oid_get_oid_by_sig_alg, oid_sig_alg_t, oid_sig_alg, mbedtls_pk_type, pk_alg, mbedtls_md_type, md_alg)

FN_OID_TYPED_FROM_ASN1(oid_pk_alg_t, pk_alg, oid_pk_alg)
FN_OID_GET_ATTR1(mbedtls_oid_get_pk_alg, oid_pk_alg_t, pk_alg, mbedtls_pk_type, pk_alg)
//FN_OID_GET_OID_BY_ATTR1(mbedtls_oid_get_oid_by_pk_alg, oid_pk_alg_t, oid_pk_alg, mbedtls_pk_type, pk_alg)



static void mbedtls_zeroize(void *v, size_t n)
{
    volatile unsigned char *p = v;
    while (n--) *p++ = 0;
}
static void mbedtls_pem_init(mbedtls_pem_context *ctx)
{
    memset(ctx, 0, sizeof(mbedtls_pem_context));
}
static int mbedtls_pem_read_buffer(mbedtls_pem_context *ctx, const char *header, const char *footer,
                                   const unsigned char *data, const unsigned char *pwd,
                                   unsigned int pwdlen, unsigned int *use_len)
{
    int ret, enc;
    unsigned int len;
    unsigned char *buf;
    const unsigned char *s1, *s2, *end;
#if defined(MBEDTLS_MD5_C) && defined(MBEDTLS_CIPHER_MODE_CBC) &&         \
    (defined(MBEDTLS_DES_C) || defined(MBEDTLS_AES_C))
    unsigned char pem_iv[16];
    mbedtls_cipher_type_t enc_alg = MBEDTLS_CIPHER_NONE;
#else
    ((void)pwd);
    ((void)pwdlen);
#endif /* MBEDTLS_MD5_C && MBEDTLS_CIPHER_MODE_CBC &&
    ( MBEDTLS_AES_C || MBEDTLS_DES_C ) */

    if (ctx == NULL)
        return(MBEDTLS_ERR_PEM_BAD_INPUT_DATA);

    s1 = (unsigned char *)strstr((const char *)data, header);

    if (s1 == NULL)
        return(MBEDTLS_ERR_PEM_NO_HEADER_FOOTER_PRESENT);

    s2 = (unsigned char *)strstr((const char *)data, footer);

    if (s2 == NULL || s2 <= s1)
        return(MBEDTLS_ERR_PEM_NO_HEADER_FOOTER_PRESENT);

    s1 += strlen(header);
    if (*s1 == ' ') s1++;
    if (*s1 == '\r') s1++;
    if (*s1 == '\n') s1++;
    else return(MBEDTLS_ERR_PEM_NO_HEADER_FOOTER_PRESENT);

    end = s2;
    end += strlen(footer);
    if (*end == ' ') end++;
    if (*end == '\r') end++;
    if (*end == '\n') end++;
    *use_len = end - data;

    enc = 0;

    if (s2 - s1 >= 22 && memcmp(s1, "Proc-Type: 4,ENCRYPTED", 22) == 0)
    {

    }

    if (s1 >= s2)
        return(MBEDTLS_ERR_PEM_INVALID_DATA);

    ret = mbedtls_base64_decode(NULL, 0, &len, s1, s2 - s1);

    if (ret == MBEDTLS_ERR_BASE64_INVALID_CHARACTER)
        return(MBEDTLS_ERR_PEM_INVALID_DATA + ret);

    if ((buf = mbedtls_calloc(1, len)) == NULL)
        return(MBEDTLS_ERR_PEM_ALLOC_FAILED);

    if ((ret = mbedtls_base64_decode(buf, len, &len, s1, s2 - s1)) != 0)
    {
        //TODO
        mbedtls_free(buf);
        return(MBEDTLS_ERR_PEM_INVALID_DATA + ret);
    }

    if (enc != 0)
    {
    }

    ctx->buf = buf;
    ctx->buflen = len;

    return(0);
}
static void mbedtls_pem_free(mbedtls_pem_context *ctx)
{

    mbedtls_free(ctx->buf);
    mbedtls_free(ctx->info);
    mbedtls_zeroize(ctx, sizeof(mbedtls_pem_context));
}


static int mbedtls_asn1_get_len(unsigned char **p,
                                const unsigned char *end,
                                size_t *len)
{
    if ((end - *p) < 1)
        return(MBEDTLS_ERR_ASN1_OUT_OF_DATA);

    if ((**p & 0x80) == 0)
        *len = *(*p)++;
    else
    {
        switch (**p & 0x7F)
        {
            case 1:
                if ((end - *p) < 2)
                    return(MBEDTLS_ERR_ASN1_OUT_OF_DATA);

                *len = (*p)[1];
                (*p) += 2;
                break;

            case 2:
                if ((end - *p) < 3)
                    return(MBEDTLS_ERR_ASN1_OUT_OF_DATA);

                *len = ((size_t)(*p)[1] << 8) | (*p)[2];
                (*p) += 3;
                break;

            case 3:
                if ((end - *p) < 4)
                    return(MBEDTLS_ERR_ASN1_OUT_OF_DATA);

                *len = ((size_t)(*p)[1] << 16) |
                       ((size_t)(*p)[2] << 8) | (*p)[3];
                (*p) += 4;
                break;

            case 4:
                if ((end - *p) < 5)
                    return(MBEDTLS_ERR_ASN1_OUT_OF_DATA);

                *len = ((size_t)(*p)[1] << 24) | ((size_t)(*p)[2] << 16) |
                       ((size_t)(*p)[3] << 8) | (*p)[4];
                (*p) += 5;
                break;

            default:
                return(MBEDTLS_ERR_ASN1_INVALID_LENGTH);
        }
    }

    if (*len > (size_t)(end - *p))
        return(MBEDTLS_ERR_ASN1_OUT_OF_DATA);

    return(0);
}

static int mbedtls_asn1_get_tag(unsigned char **p,
                                const unsigned char *end,
                                size_t *len, int tag)
{
    if ((end - *p) < 1)
        return(MBEDTLS_ERR_ASN1_OUT_OF_DATA);

    if (**p != tag)
        return(MBEDTLS_ERR_ASN1_UNEXPECTED_TAG);

    (*p)++;

    return(mbedtls_asn1_get_len(p, end, len));
}

static int mbedtls_x509_get_ext(unsigned char **p, const unsigned char *end,
                                mbedtls_x509_buf *ext, int tag)
{
    int ret;
    size_t len;

    if (*p == end)
        return(0);

    ext->tag = **p;

    if ((ret = mbedtls_asn1_get_tag(p, end, &ext->len,
                                    MBEDTLS_ASN1_CONTEXT_SPECIFIC | MBEDTLS_ASN1_CONSTRUCTED | tag)) != 0)
        return(ret);

    ext->p = *p;
    end = *p + ext->len;

    /*
    * Extensions  ::=  SEQUENCE SIZE (1..MAX) OF Extension
    *
    * Extension  ::=  SEQUENCE  {
    *      extnID      OBJECT IDENTIFIER,
    *      critical    BOOLEAN DEFAULT FALSE,
    *      extnValue   OCTET STRING  }
    */
    if ((ret = mbedtls_asn1_get_tag(p, end, &len,
                                    MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE)) != 0)
        return(MBEDTLS_ERR_X509_INVALID_EXTENSIONS + ret);

    if (end != *p + len)
        return(MBEDTLS_ERR_X509_INVALID_EXTENSIONS +
               MBEDTLS_ERR_ASN1_LENGTH_MISMATCH);

    return(0);
}


static int x509_parse_int(unsigned char **p, size_t n, int *res)
{
    *res = 0;

    for (; n > 0; --n)
    {
        if ((**p < '0') || (**p > '9'))
            return (MBEDTLS_ERR_X509_INVALID_DATE);

        *res *= 10;
        *res += (*(*p)++ - '0');
    }

    return(0);
}

static int x509_date_is_valid(const mbedtls_x509_time *time)
{
    int ret = MBEDTLS_ERR_X509_INVALID_DATE;

    CHECK_RANGE(0, 9999, time->year);
    CHECK_RANGE(0, 23, time->hour);
    CHECK_RANGE(0, 59, time->min);
    CHECK_RANGE(0, 59, time->sec);

    switch (time->mon)
    {
        case 1:
        case 3:
        case 5:
        case 7:
        case 8:
        case 10:
        case 12:
            CHECK_RANGE(1, 31, time->day);
            break;
        case 4:
        case 6:
        case 9:
        case 11:
            CHECK_RANGE(1, 30, time->day);
            break;
        case 2:
            CHECK_RANGE(1, 28 + (time->year % 4 == 0), time->day);
            break;
        default:
            return(ret);
    }

    return(0);
}


static int x509_parse_time(unsigned char **p, size_t len, size_t yearlen,
                           mbedtls_x509_time *time)
{
    int ret;

    /*
    * Minimum length is 10 or 12 depending on yearlen
    */
    if (len < yearlen + 8)
        return (MBEDTLS_ERR_X509_INVALID_DATE);
    len -= yearlen + 8;

    /*
    * Parse year, month, day, hour, minute
    */
    CHECK(x509_parse_int(p, yearlen, &time->year));
    if (2 == yearlen)
    {
        if (time->year < 50)
            time->year += 100;

        time->year += 1900;
    }

    CHECK(x509_parse_int(p, 2, &time->mon));
    CHECK(x509_parse_int(p, 2, &time->day));
    CHECK(x509_parse_int(p, 2, &time->hour));
    CHECK(x509_parse_int(p, 2, &time->min));

    /*
    * Parse seconds if present
    */
    if (len >= 2)
    {
        CHECK(x509_parse_int(p, 2, &time->sec));
        len -= 2;
    }
    else
        return (MBEDTLS_ERR_X509_INVALID_DATE);

    /*
    * Parse trailing 'Z' if present
    */
    if (1 == len && 'Z' == **p)
    {
        (*p)++;
        len--;
    }

    /*
    * We should have parsed all characters at this point
    */
    if (0 != len)
        return (MBEDTLS_ERR_X509_INVALID_DATE);

    CHECK(x509_date_is_valid(time));

    return (0);
}


static int mbedtls_x509_get_time(unsigned char **p, const unsigned char *end,
                                 mbedtls_x509_time *time)
{
    int ret;
    size_t len, year_len;
    unsigned char tag;

    if ((end - *p) < 1)
        return(MBEDTLS_ERR_X509_INVALID_DATE +
               MBEDTLS_ERR_ASN1_OUT_OF_DATA);

    tag = **p;

    if (tag == MBEDTLS_ASN1_UTC_TIME)
        year_len = 2;
    else if (tag == MBEDTLS_ASN1_GENERALIZED_TIME)
        year_len = 4;
    else
        return(MBEDTLS_ERR_X509_INVALID_DATE +
               MBEDTLS_ERR_ASN1_UNEXPECTED_TAG);

    (*p)++;
    ret = mbedtls_asn1_get_len(p, end, &len);

    if (ret != 0)
        return(MBEDTLS_ERR_X509_INVALID_DATE + ret);

    return x509_parse_time(p, len, year_len, time);
}

static int mbedtls_x509_get_serial(unsigned char **p, const unsigned char *end,
                                   mbedtls_x509_buf *serial)
{
    int ret;

    if ((end - *p) < 1)
        return(MBEDTLS_ERR_X509_INVALID_SERIAL +
               MBEDTLS_ERR_ASN1_OUT_OF_DATA);

    if (**p != (MBEDTLS_ASN1_CONTEXT_SPECIFIC | MBEDTLS_ASN1_PRIMITIVE | 2) &&
        **p != MBEDTLS_ASN1_INTEGER)
        return(MBEDTLS_ERR_X509_INVALID_SERIAL +
               MBEDTLS_ERR_ASN1_UNEXPECTED_TAG);

    serial->tag = *(*p)++;

    if ((ret = mbedtls_asn1_get_len(p, end, &serial->len)) != 0)
        return(MBEDTLS_ERR_X509_INVALID_SERIAL + ret);

    serial->p = *p;
    *p += serial->len;

    return(0);
}

static int mbedtls_x509_get_alg(unsigned char **p, const unsigned char *end,
                                mbedtls_x509_buf *alg, mbedtls_x509_buf *params)
{
    int ret;
    size_t len;

    if ((ret = mbedtls_asn1_get_tag(p, end, &len,
                                    MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE)) != 0)
        return(ret);

    if ((end - *p) < 1)
        return(MBEDTLS_ERR_ASN1_OUT_OF_DATA);

    alg->tag = **p;
    end = *p + len;

    if ((ret = mbedtls_asn1_get_tag(p, end, &alg->len, MBEDTLS_ASN1_OID)) != 0)
        return(ret);

    alg->p = *p;
    *p += alg->len;

    if (*p == end)
    {
        mbedtls_zeroize(params, sizeof(mbedtls_asn1_buf));
        return(0);
    }

    params->tag = **p;
    (*p)++;

    if ((ret = mbedtls_asn1_get_len(p, end, &params->len)) != 0)
        return(ret);

    params->p = *p;
    *p += params->len;

    if (*p != end)
        return(MBEDTLS_ERR_ASN1_LENGTH_MISMATCH);

    return(0);
}

static int mbedtls_asn1_get_bitstring_null(unsigned char **p, const unsigned char *end,
        size_t *len)
{
    int ret;

    if ((ret = mbedtls_asn1_get_tag(p, end, len, MBEDTLS_ASN1_BIT_STRING)) != 0)
        return(ret);

    if ((*len)-- < 2 || *(*p)++ != 0)
        return(MBEDTLS_ERR_ASN1_INVALID_DATA);

    return(0);
}


static int mbedtls_x509_get_sig(unsigned char **p, const unsigned char *end, mbedtls_x509_buf *sig)
{
    int ret;
    size_t len;
    int tag_type;

    if ((end - *p) < 1)
        return(MBEDTLS_ERR_X509_INVALID_SIGNATURE +
               MBEDTLS_ERR_ASN1_OUT_OF_DATA);

    tag_type = **p;

    if ((ret = mbedtls_asn1_get_bitstring_null(p, end, &len)) != 0)
        return(MBEDTLS_ERR_X509_INVALID_SIGNATURE + ret);

    sig->tag = tag_type;
    sig->len = len;
    sig->p = *p;

    *p += len;

    return(0);
}

static int mbedtls_x509_get_sig_alg(const mbedtls_x509_buf *sig_oid, const mbedtls_x509_buf *sig_params,
                                    mbedtls_md_type *md_alg, mbedtls_pk_type *pk_alg)
{
    int ret;

    if ((ret = mbedtls_oid_get_sig_alg(sig_oid, md_alg, pk_alg)) != 0)
        return(MBEDTLS_ERR_X509_UNKNOWN_SIG_ALG + ret);

    {
        /* Make sure parameters are absent or NULL */
        if ((sig_params->tag != MBEDTLS_ASN1_NULL && sig_params->tag != 0) ||
            sig_params->len != 0)
            return(MBEDTLS_ERR_X509_INVALID_ALG);
    }

    return(0);
}

static int x509_get_attr_type_value(unsigned char **p,
                                    const unsigned char *end,
                                    mbedtls_x509_name *cur)
{
    int ret;
    size_t len;
    mbedtls_x509_buf *oid;
    mbedtls_x509_buf *val;

    if ((ret = mbedtls_asn1_get_tag(p, end, &len,
                                    MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE)) != 0)
        return(MBEDTLS_ERR_X509_INVALID_NAME + ret);

    if ((end - *p) < 1)
        return(MBEDTLS_ERR_X509_INVALID_NAME +
               MBEDTLS_ERR_ASN1_OUT_OF_DATA);

    oid = &cur->oid;
    oid->tag = **p;

    if ((ret = mbedtls_asn1_get_tag(p, end, &oid->len, MBEDTLS_ASN1_OID)) != 0)
        return(MBEDTLS_ERR_X509_INVALID_NAME + ret);

    oid->p = *p;
    *p += oid->len;

    if ((end - *p) < 1)
        return(MBEDTLS_ERR_X509_INVALID_NAME +
               MBEDTLS_ERR_ASN1_OUT_OF_DATA);

    if (**p != MBEDTLS_ASN1_BMP_STRING && **p != MBEDTLS_ASN1_UTF8_STRING      &&
        **p != MBEDTLS_ASN1_T61_STRING && **p != MBEDTLS_ASN1_PRINTABLE_STRING &&
        **p != MBEDTLS_ASN1_IA5_STRING && **p != MBEDTLS_ASN1_UNIVERSAL_STRING &&
        **p != MBEDTLS_ASN1_BIT_STRING)
        return(MBEDTLS_ERR_X509_INVALID_NAME +
               MBEDTLS_ERR_ASN1_UNEXPECTED_TAG);

    val = &cur->val;
    val->tag = *(*p)++;

    if ((ret = mbedtls_asn1_get_len(p, end, &val->len)) != 0)
        return(MBEDTLS_ERR_X509_INVALID_NAME + ret);

    val->p = *p;
    *p += val->len;

    cur->next = NULL;

    return(0);
}

static int mbedtls_x509_get_name(unsigned char **p, const unsigned char *end,
                                 mbedtls_x509_name *cur)
{
    int ret;
    size_t set_len;
    const unsigned char *end_set;

    /* don't use recursion, we'd risk stack overflow if not optimized */
    while (1)
    {
        /*
        * parse SET
        */
        if ((ret = mbedtls_asn1_get_tag(p, end, &set_len,
                                        MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SET)) != 0)
            return(MBEDTLS_ERR_X509_INVALID_NAME + ret);

        end_set = *p + set_len;

        while (1)
        {
            if ((ret = x509_get_attr_type_value(p, end_set, cur)) != 0)
                return(ret);

            if (*p == end_set)
                break;

            /* Mark this item as being no the only one in a set */
            cur->next_merged = 1;

            cur->next = mbedtls_calloc(1, sizeof(mbedtls_x509_name));

            if (cur->next == NULL)
                return(MBEDTLS_ERR_X509_ALLOC_FAILED);

            cur = cur->next;
        }

        /*
        * continue until end of SEQUENCE is reached
        */
        if (*p == end)
            return(0);

        cur->next = mbedtls_calloc(1, sizeof(mbedtls_x509_name));

        if (cur->next == NULL)
            return(MBEDTLS_ERR_X509_ALLOC_FAILED);

        cur = cur->next;
    }
}

static int mbedtls_asn1_get_alg(unsigned char **p,
                                const unsigned char *end,
                                mbedtls_asn1_buf *alg, mbedtls_asn1_buf *params)
{
    int ret;
    size_t len;

    if ((ret = mbedtls_asn1_get_tag(p, end, &len,
                                    MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE)) != 0)
        return(ret);

    if ((end - *p) < 1)
        return(MBEDTLS_ERR_ASN1_OUT_OF_DATA);

    alg->tag = **p;
    end = *p + len;

    if ((ret = mbedtls_asn1_get_tag(p, end, &alg->len, MBEDTLS_ASN1_OID)) != 0)
        return(ret);

    alg->p = *p;
    *p += alg->len;

    if (*p == end)
    {
        mbedtls_zeroize(params, sizeof(mbedtls_asn1_buf));
        return(0);
    }

    params->tag = **p;
    (*p)++;

    if ((ret = mbedtls_asn1_get_len(p, end, &params->len)) != 0)
        return(ret);

    params->p = *p;
    *p += params->len;

    if (*p != end)
        return(MBEDTLS_ERR_ASN1_LENGTH_MISMATCH);

    return(0);
}



static int pk_get_pk_alg(unsigned char **p,
                         const unsigned char *end,
                         mbedtls_pk_type *pk_alg, mbedtls_asn1_buf *params)
{
    int ret;
    mbedtls_asn1_buf alg_oid;

    memset(params, 0, sizeof(mbedtls_asn1_buf));

    if ((ret = mbedtls_asn1_get_alg(p, end, &alg_oid, params)) != 0)
        return(ret);

    if (mbedtls_oid_get_pk_alg(&alg_oid, pk_alg) != 0)
        return(MBEDTLS_ERR_PK_UNKNOWN_PK_ALG);

    /*
    * No parameters with RSA (only for EC)
    */
    if (*pk_alg == MBEDTLS_PK_RSA &&
        ((params->tag != MBEDTLS_ASN1_NULL && params->tag != 0) ||
         params->len != 0))
    {
        return(MBEDTLS_ERR_PK_INVALID_ALG);
    }

    return(0);
}

static int mbedtls_x509_get_pubkey(unsigned char **p, const unsigned char *end,
                                   mbedtls_x509_buf *pk)
{
    int ret;
    size_t len;
    mbedtls_asn1_buf alg_params;
    mbedtls_pk_type pk_alg = MBEDTLS_PK_NONE;

    if ((ret = mbedtls_asn1_get_tag(p, end, &len,
                                    MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE)) != 0)
    {
        return(ret);
    }

    end = *p + len;

    if ((ret = pk_get_pk_alg(p, end, &pk_alg, &alg_params)) != 0)
        return(ret);

    if ((ret = mbedtls_asn1_get_bitstring_null(p, end, &len)) != 0)
        return(MBEDTLS_ERR_PK_INVALID_PUBKEY + ret);

    if (*p + len != end)
        return(MBEDTLS_ERR_PK_INVALID_PUBKEY +
               MBEDTLS_ERR_ASN1_LENGTH_MISMATCH);


    pk->len = len;
    pk->tag = *(*p)++;
    pk->p = *p;
    *p = (unsigned char *)end;
    return(ret);
}


static int mbedtls_asn1_get_int(unsigned char **p,
                                const unsigned char *end,
                                int *val)
{
    int ret;
    size_t len;

    if ((ret = mbedtls_asn1_get_tag(p, end, &len, MBEDTLS_ASN1_INTEGER)) != 0)
        return(ret);

    if (len == 0 || len > sizeof(int) || (**p & 0x80) != 0)
        return(MBEDTLS_ERR_ASN1_INVALID_LENGTH);

    *val = 0;

    while (len-- > 0)
    {
        *val = (*val << 8) | **p;
        (*p)++;
    }

    return(0);
}


static int x509_get_version(unsigned char **p,
                            const unsigned char *end,
                            int *ver)
{
    int ret;
    size_t len;

    if ((ret = mbedtls_asn1_get_tag(p, end, &len,
                                    MBEDTLS_ASN1_CONTEXT_SPECIFIC | MBEDTLS_ASN1_CONSTRUCTED | 0)) != 0)
    {
        if (ret == MBEDTLS_ERR_ASN1_UNEXPECTED_TAG)
        {
            *ver = 0;
            return(0);
        }

        return(ret);
    }

    end = *p + len;

    if ((ret = mbedtls_asn1_get_int(p, end, ver)) != 0)
        return(MBEDTLS_ERR_X509_INVALID_VERSION + ret);

    if (*p != end)
        return(MBEDTLS_ERR_X509_INVALID_VERSION +
               MBEDTLS_ERR_ASN1_LENGTH_MISMATCH);

    return(0);
}

static int x509_get_dates(unsigned char **p,
                          const unsigned char *end,
                          mbedtls_x509_time *from,
                          mbedtls_x509_time *to)
{
    int ret;
    size_t len;

    if ((ret = mbedtls_asn1_get_tag(p, end, &len,
                                    MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE)) != 0)
        return(MBEDTLS_ERR_X509_INVALID_DATE + ret);

    end = *p + len;

    if ((ret = mbedtls_x509_get_time(p, end, from)) != 0)
        return(ret);

    if ((ret = mbedtls_x509_get_time(p, end, to)) != 0)
        return(ret);

    if (*p != end)
        return(MBEDTLS_ERR_X509_INVALID_DATE +
               MBEDTLS_ERR_ASN1_LENGTH_MISMATCH);

    return(0);
}


static int x509_get_uid(unsigned char **p,
                        const unsigned char *end,
                        mbedtls_x509_buf *uid, int n)
{
    int ret;

    if (*p == end)
        return(0);

    uid->tag = **p;

    if ((ret = mbedtls_asn1_get_tag(p, end, &uid->len,
                                    MBEDTLS_ASN1_CONTEXT_SPECIFIC | MBEDTLS_ASN1_CONSTRUCTED | n)) != 0)
    {
        if (ret == MBEDTLS_ERR_ASN1_UNEXPECTED_TAG)
            return(0);

        return(ret);
    }

    uid->p = *p;
    *p += uid->len;

    return(0);
}


static int x509_get_crt_ext(unsigned char **p,
                            const unsigned char *end,
                            mbedtls_x509_crt *crt)
{
    int ret;
    size_t len;

    if ((ret = mbedtls_x509_get_ext(p, end, &crt->v3_ext, 3)) != 0)
    {
        if (ret == MBEDTLS_ERR_ASN1_UNEXPECTED_TAG)
            return(0);

        return(ret);
    }

    while (*p < end)
    {


        if ((ret = mbedtls_asn1_get_tag(p, end, &len,
                                        MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE)) != 0)
            return(MBEDTLS_ERR_X509_INVALID_EXTENSIONS + ret);
        // ֱӻȡ UID
        *p += len;
        /*if ((ret = mbedtls_asn1_get_tag(p, end, &len,
        MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE)) != 0)
        return(MBEDTLS_ERR_X509_INVALID_EXTENSIONS + ret);*/

        /*ret = x509_get_uid(p, end, &crt->issuer_id, 1);
        if (ret != 0)
        {
        pos_x509_crt_free(crt);
        return(ret);
        }*/
        *p = end;

    }
    return(0);
}



int pos_x509_crt_parse_der(mbedtls_x509_crt *crt, const unsigned char *buf,
                           size_t buflen)
{
    int ret;
    size_t len;
    unsigned char *p, *end, *crt_end;
    mbedtls_x509_buf sig_params1, sig_params2, sig_oid2;

    memset(&sig_params1, 0, sizeof(mbedtls_x509_buf));
    memset(&sig_params2, 0, sizeof(mbedtls_x509_buf));
    memset(&sig_oid2, 0, sizeof(mbedtls_x509_buf));

    /*
    * Check for valid input
    */
    if (crt == NULL || buf == NULL)
        return(MBEDTLS_ERR_X509_BAD_INPUT_DATA);

    // Use the original buffer until we figure out actual length
    p = (unsigned char*)buf;
    len = buflen;
    end = p + len;

    /*
    * Certificate  ::=  SEQUENCE  {
    *      tbsCertificate       TBSCertificate,
    *      signatureAlgorithm   AlgorithmIdentifier,
    *      signatureValue       BIT STRING  }
    */
    if ((ret = mbedtls_asn1_get_tag(&p, end, &len,
                                    MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE)) != 0)
    {
        pos_x509_crt_free(crt);
        return(MBEDTLS_ERR_X509_INVALID_FORMAT);
    }

    if (len > (size_t)(end - p))
    {
        pos_x509_crt_free(crt);
        return(MBEDTLS_ERR_X509_INVALID_FORMAT +
               MBEDTLS_ERR_ASN1_LENGTH_MISMATCH);
    }
    crt_end = p + len;

    // Create and populate a new buffer for the raw field
    crt->raw.len = crt_end - buf;
    crt->raw.p = p = mbedtls_calloc(1, crt->raw.len);
    if (p == NULL)
        return(MBEDTLS_ERR_X509_ALLOC_FAILED);

    memcpy(p, buf, crt->raw.len);

    // Direct pointers to the new buffer
    p += crt->raw.len - len;
    end = crt_end = p + len;

    /*
    * TBSCertificate  ::=  SEQUENCE  {
    */
    crt->tbs.p = p;

    if ((ret = mbedtls_asn1_get_tag(&p, end, &len,
                                    MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE)) != 0)
    {
        pos_x509_crt_free(crt);
        return(MBEDTLS_ERR_X509_INVALID_FORMAT + ret);
    }

    end = p + len;
    crt->tbs.len = end - crt->tbs.p;

    /*
    * Version  ::=  INTEGER  {  v1(0), v2(1), v3(2)  }
    *
    * CertificateSerialNumber  ::=  INTEGER
    *
    * signature            AlgorithmIdentifier
    */
    if ((ret = x509_get_version(&p, end, &crt->version)) != 0 ||
        (ret = mbedtls_x509_get_serial(&p, end, &crt->serial)) != 0 ||
        (ret = mbedtls_x509_get_alg(&p, end, &crt->sig_oid,
                                    &sig_params1)) != 0)
    {
        pos_x509_crt_free(crt);
        return(ret);
    }

    crt->version++;

    if (crt->version > 3)
    {
        pos_x509_crt_free(crt);
        return(MBEDTLS_ERR_X509_UNKNOWN_VERSION);
    }

    if ((ret = mbedtls_x509_get_sig_alg(&crt->sig_oid, &sig_params1,
                                        &crt->sig_md, &crt->sig_pk)) != 0)
    {
        pos_x509_crt_free(crt);
        return(ret);
    }

    /*
    * issuer               Name
    */
    crt->issuer_raw.p = p;

    if ((ret = mbedtls_asn1_get_tag(&p, end, &len,
                                    MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE)) != 0)
    {
        pos_x509_crt_free(crt);
        return(MBEDTLS_ERR_X509_INVALID_FORMAT + ret);
    }

    if ((ret = mbedtls_x509_get_name(&p, p + len, &crt->issuer)) != 0)
    {
        pos_x509_crt_free(crt);
        return(ret);
    }

    crt->issuer_raw.len = p - crt->issuer_raw.p;

    /*
    * Validity ::= SEQUENCE {
    *      notBefore      Time,
    *      notAfter       Time }
    *
    */
    if ((ret = x509_get_dates(&p, end, &crt->valid_from,
                              &crt->valid_to)) != 0)
    {
        pos_x509_crt_free(crt);
        return(ret);
    }

    /*
    * subject              Name
    */
    crt->subject_raw.p = p;

    if ((ret = mbedtls_asn1_get_tag(&p, end, &len,
                                    MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE)) != 0)
    {
        pos_x509_crt_free(crt);
        return(MBEDTLS_ERR_X509_INVALID_FORMAT + ret);
    }

    if (len && (ret = mbedtls_x509_get_name(&p, p + len, &crt->subject)) != 0)
    {
        pos_x509_crt_free(crt);
        return(ret);
    }

    crt->subject_raw.len = p - crt->subject_raw.p;

    /*
    * SubjectPublicKeyInfo
    */
    if ((ret = mbedtls_x509_get_pubkey(&p, end, &crt->pk)) != 0)
    {
        pos_x509_crt_free(crt);
        return(ret);
    }

    if (crt->version == 2 || crt->version == 3)
    {
        ret = x509_get_uid(&p, end, &crt->issuer_id, 1);
        if (ret != 0)
        {
            pos_x509_crt_free(crt);
            return(ret);
        }
    }

    if (crt->version == 2 || crt->version == 3)
    {
        ret = x509_get_uid(&p, end, &crt->subject_id, 2);
        if (ret != 0)
        {
            pos_x509_crt_free(crt);
            return(ret);
        }
    }

    if (crt->version == 3)
    {
        ret = x509_get_crt_ext(&p, end, crt);
        if (ret != 0)
        {
            pos_x509_crt_free(crt);
            return(ret);
        }
    }

    if (p != end)
    {
        pos_x509_crt_free(crt);
        return(MBEDTLS_ERR_X509_INVALID_FORMAT +
               MBEDTLS_ERR_ASN1_LENGTH_MISMATCH);
    }

    end = crt_end;

    /*
    *  }
    *  -- end of TBSCertificate
    *
    *  signatureAlgorithm   AlgorithmIdentifier,
    *  signatureValue       BIT STRING
    */
    if ((ret = mbedtls_x509_get_alg(&p, end, &sig_oid2, &sig_params2)) != 0)
    {
        pos_x509_crt_free(crt);
        return(ret);
    }

    if (crt->sig_oid.len != sig_oid2.len ||
        memcmp(crt->sig_oid.p, sig_oid2.p, crt->sig_oid.len) != 0 ||
        sig_params1.len != sig_params2.len ||
        (sig_params1.len != 0 &&
         memcmp(sig_params1.p, sig_params2.p, sig_params1.len) != 0))
    {
        pos_x509_crt_free(crt);
        return(MBEDTLS_ERR_X509_SIG_MISMATCH);
    }

    if ((ret = mbedtls_x509_get_sig(&p, end, &crt->sig)) != 0)
    {
        pos_x509_crt_free(crt);
        return(ret);
    }

    if (p != end)
    {
        pos_x509_crt_free(crt);
        return(MBEDTLS_ERR_X509_INVALID_FORMAT +
               MBEDTLS_ERR_ASN1_LENGTH_MISMATCH);
    }


    return(0);
}


int pos_x509_crt_parse(mbedtls_x509_crt *chain, const unsigned char *buf, unsigned int buflen)
{
    int success = 0, first_error = 0, total_failed = 0;
    int buf_format = MBEDTLS_X509_FORMAT_DER;

    if (chain == NULL || buf == NULL)
        return(MBEDTLS_ERR_X509_BAD_INPUT_DATA);

    /*
    * Determine buffer content. Buffer contains either one DER certificate or
    * one or more PEM certificates.
    */
    if (buflen != 0 && buf[buflen - 1] == '\0' &&
        strstr((const char *)buf, "-----BEGIN CERTIFICATE-----") != NULL)
    {
        buf_format = MBEDTLS_X509_FORMAT_PEM;
    }

    if (buf_format == MBEDTLS_X509_FORMAT_DER)
        return pos_x509_crt_parse_der(chain, buf, buflen);

    if (buf_format == MBEDTLS_X509_FORMAT_PEM)
    {
        int ret = 0;
        mbedtls_pem_context pem;

        /* 1 rather than 0 since the terminating NULL byte is counted in */
        while (buflen > 1)
        {
            unsigned int use_len = 0;
            mbedtls_pem_init(&pem);

            /* If we get there, we know the string is null-terminated */
            ret = mbedtls_pem_read_buffer(&pem,
                                          "-----BEGIN CERTIFICATE-----",
                                          "-----END CERTIFICATE-----",
                                          buf, NULL, 0, &use_len);

            if (ret == 0)
            {
                /*
                * Was PEM encoded
                */
                buflen -= use_len;
                buf += use_len;
            }
            else if (ret == MBEDTLS_ERR_PEM_BAD_INPUT_DATA)
            {
                return(ret);
            }
            else if (ret != MBEDTLS_ERR_PEM_NO_HEADER_FOOTER_PRESENT)
            {
                mbedtls_pem_free(&pem);

                /*
                * PEM header and footer were found
                */
                buflen -= use_len;
                buf += use_len;

                if (first_error == 0)
                    first_error = ret;

                total_failed++;
                continue;
            }
            else
                break;

            ret = pos_x509_crt_parse_der(chain, pem.buf, pem.buflen);

            mbedtls_pem_free(&pem);

            if (ret != 0)
            {
                /*
                * Quit parsing on a memory error
                */
                if (ret == MBEDTLS_ERR_X509_ALLOC_FAILED)
                    return(ret);

                if (first_error == 0)
                    first_error = ret;

                total_failed++;
                continue;
            }

            success = 1;
        }
    }

    if (success)
        return(total_failed);
    else if (first_error)
        return(first_error);
    else
        return(MBEDTLS_ERR_X509_CERT_UNKNOWN_FORMAT);
}


void pos_x509_crt_init(mbedtls_x509_crt *crt)
{
    memset(crt, 0, sizeof(mbedtls_x509_crt));
}


void pos_x509_crt_free(mbedtls_x509_crt *crt)
{
    mbedtls_x509_crt *cert_cur = crt;
    mbedtls_x509_crt *cert_prv;
    mbedtls_x509_name *name_cur;
    mbedtls_x509_name *name_prv;
    mbedtls_x509_sequence *seq_cur;
    mbedtls_x509_sequence *seq_prv;

    if (crt == NULL)
        return;

    do
    {

#if defined(MBEDTLS_X509_RSASSA_PSS_SUPPORT)
        mbedtls_free(cert_cur->sig_opts);
#endif

        name_cur = cert_cur->issuer.next;
        while (name_cur != NULL)
        {
            name_prv = name_cur;
            name_cur = name_cur->next;
            mbedtls_zeroize(name_prv, sizeof(mbedtls_x509_name));
            mbedtls_free(name_prv);
        }

        name_cur = cert_cur->subject.next;
        while (name_cur != NULL)
        {
            name_prv = name_cur;
            name_cur = name_cur->next;
            mbedtls_zeroize(name_prv, sizeof(mbedtls_x509_name));
            mbedtls_free(name_prv);
        }

        seq_cur = cert_cur->ext_key_usage.next;
        while (seq_cur != NULL)
        {
            seq_prv = seq_cur;
            seq_cur = seq_cur->next;
            mbedtls_zeroize(seq_prv, sizeof(mbedtls_x509_sequence));
            mbedtls_free(seq_prv);
        }

        seq_cur = cert_cur->subject_alt_names.next;
        while (seq_cur != NULL)
        {
            seq_prv = seq_cur;
            seq_cur = seq_cur->next;
            mbedtls_zeroize(seq_prv, sizeof(mbedtls_x509_sequence));
            mbedtls_free(seq_prv);
        }

        if (cert_cur->raw.p != NULL)
        {
            mbedtls_zeroize(cert_cur->raw.p, cert_cur->raw.len);
            mbedtls_free(cert_cur->raw.p);
        }

        cert_cur = NULL;
    }
    while (cert_cur != NULL);

    cert_cur = crt;
    do
    {
        cert_prv = cert_cur;
        mbedtls_zeroize(cert_prv, sizeof(mbedtls_x509_crt));
        if (cert_prv != crt)
            mbedtls_free(cert_prv);
        cert_cur = NULL;
    }
    while (cert_cur != NULL);
}

