#include "bwsdk_api.h"
#include "mmi_util.h"

#define MMI_PARAM_FILE  "mmi_param.ini"


typedef struct DISPLAY_PARA_STRUCT
{
    tsScrInfo       srcSize;

    int             fontSize;           //С(ʱò,չ)
    int             fontColor;          //ɫ

    int             backgroundColor;    //ɫ
    char            *backgroundImg;

    int             progressBar_backgroundColor;    //ɫ
    int             progressBar_borderColor;        //߿ɫ
} DISPLAY_PARA;

/**
 * @brief ʱʾ
 */
typedef struct DISPLAY_TIMER_PARA_STRUCT
{
    OFF_ON          flipEnable;
    unsigned char   interrupt;
    TIMER_PTR       timer;
    int             time;
    int             timeout;
    int             line;
    int             column;
} DISPLAY_TIMER_PARA;


static DISPLAY_PARA s_mmi_lcd_param;
static DISPLAY_TIMER_PARA s_mmi_timer_param = {MMI_DISABLED, 0, NULL, 0, 0, 0, 0};


static int __mmi_display(unsigned int line, int fontColor, int backgroundColor, OFF_ON backgroundEnable, const char *dispBuf, int maxDispLen);
static int __mmi_displayEx(unsigned int line, unsigned int column, int fontColor, int backgroundColor, OFF_ON backgroundEnable, char *dispString);

/**
 * @brief Ĭϲ
 */
static void mmi_loadDefParam(void)
{
    memset(&s_mmi_lcd_param, 0, sizeof(s_mmi_lcd_param));
    ScrGetStatus(&s_mmi_lcd_param.srcSize);
    if (FsSize(MMI_PARAM_FILE) <= 0)
    {
        //
        s_mmi_lcd_param.fontColor = MERCURY_BLACK;

        //
        s_mmi_lcd_param.backgroundColor = MERCURY_WHITE;

        //
        s_mmi_lcd_param.progressBar_backgroundColor = MERCURY_BLUE;
        s_mmi_lcd_param.progressBar_borderColor = MERCURY_BLACK;
        return;
    }

    //
    fileCfgToolReadKeyInt("font", "size", MMI_PARAM_FILE, &s_mmi_lcd_param.fontSize);
    fileCfgToolReadKeyInt("font", "color", MMI_PARAM_FILE, &s_mmi_lcd_param.fontColor);
    fileCfgToolReadKeyInt("font", "bgcolor", MMI_PARAM_FILE, &s_mmi_lcd_param.fontColor);

    //
    fileCfgToolReadKeyInt("background", "color", MMI_PARAM_FILE, &s_mmi_lcd_param.backgroundColor);

    //
    fileCfgToolReadKeyInt("progressBar", "bgcolor", MMI_PARAM_FILE, &s_mmi_lcd_param.progressBar_backgroundColor);
    fileCfgToolReadKeyInt("progressBar", "border_color", MMI_PARAM_FILE, &s_mmi_lcd_param.progressBar_borderColor);

    //״̬
}

/**
 * @brief Ҫʾλ
 */
static unsigned int mmi_get_display_line(unsigned int line)
{
    switch(line)
    {
        case ALIG_TOP:
            line = 1;
            break;
        case ALIG_BOTTOM:
            line = s_mmi_lcd_param.srcSize.Height-ScrGetStatusBarHeight();
            line = (line/LINEPIX);// + ((line%LINEPIX) ? 1 : 0);
            break;
        case ALIG_CENTER:
            line = (s_mmi_lcd_param.srcSize.Height-LINEPIX-ScrGetStatusBarHeight()) / 2;
            line /= LINEPIX;
            break;
        default:
            break;
    }
    return line;
}

/**
 * @brief Ļʾ͸ʽÿγʱһμ
 */
void mmi_init(int fontColor, int backgroundColor, char *bgImgName)
{
    mmi_loadDefParam();
    s_mmi_lcd_param.backgroundColor = backgroundColor;
    s_mmi_lcd_param.fontColor = fontColor;
    if (bgImgName)
    {
        s_mmi_lcd_param.backgroundImg = bgImgName;
    }
}

/**
 * @brief ʾɫ
 */
static void mmi_set_font(unsigned char size, unsigned char type, int fColor, int bColor)
{
    tsFont pFont;

    pFont.Type = type;  //Ĭ/0
    pFont.Size = size;
    if(bColor != MERCURY_NONE)
    {
        pFont.Mode = NON_TRANSPARENT;
    }
    else
    {
        pFont.Mode = TRANSPARENT;
    }
    pFont.fColor = fColor;
    pFont.bColor = bColor;
    ScrSetFont(&pFont);
}

/**
 * @brief ʾɫ
 * @param [in] line ʾ(ʹ 뷽ʽ ö)
 * @param [in] column ʾ(ʹ 뷽ʽ ö)
 * @param [in] backgroundColor ɫ
 */
static void mmi_set_background(unsigned int line, int backgroundColor)
{
    tsRect rect = {0};

    rect.Left = 0;
    rect.Top = LINEPIX * line;
    rect.Width = s_mmi_lcd_param.srcSize.Width;
    rect.Height = LINEPIX;
    ScrFillRect(&rect, backgroundColor);
}

/**
 * @brief Ļϵָ
 * @param [in] line 1-9, 255
 */
void mmi_clearLine(unsigned int line)
{
    tsRect rect = {0};

    if (line == 255)
    {
        ScrCls();
        return;
    }

    if (line != ALIG_TOP
        && line != ALIG_BOTTOM
        && line != ALIG_CENTER)
    {
        if((LINEPIX * line) >= (s_mmi_lcd_param.srcSize.Height-FONTPIX))
        {
            return;
        }
    }

    line = mmi_get_display_line(line);
    if(((LINEPIX * line) < (s_mmi_lcd_param.srcSize.Height-FONTPIX)))
    {
        rect.Width = s_mmi_lcd_param.srcSize.Width;
        rect.Top = line * LINEPIX;
        rect.Left = 0;
        rect.Height = FONTPIX + ((LINEPIX - FONTPIX) / 2);
        ScrRectClear(&rect);
    }
}

/**
 * @brief ָĻϵָһıϢ(ֻ֧)
 * @param [in] line ʾ(ʹ 뷽ʽ ö)
 * @param [in] fontColor ɫ
 * @param [in] backgroundColor ɫ
 * @param [in] backgroundEnable е(MMI_ENABLED-ɫʾ, MMI_DISABLED-ֻʾ屳ɫ)
 * @param [in] dispString ʾ
 * @param [in] maxDispLen ʾ󳤶
 * @retval һе
 */
int __mmi_display(unsigned int line, int fontColor, int backgroundColor, OFF_ON backgroundEnable, const char *dispString, int maxDispLen)
{
    char m_dispBuf[SCREEN_WIDTH+1];
    char *pBuf;
    int m_line = 0;
    int i, m_dispLen;
    unsigned int m_ascCount;

    pBuf = (char *)dispString;
    m_dispLen = strlen((char *) dispString);

    m_ascCount = 0;
    m_line = line;
    for (i = 0; i < m_dispLen; i++)
    {
        //cǷΪASCIIַ,ҲжcķΧǷ0127֮
        if ((pBuf[i] >= 0x80 && pBuf[i] <= 0xFE))
        {
            m_ascCount++;
        }

        if ((i + 1) % maxDispLen == 0)    //һжֽ
        {
            memset(m_dispBuf, 0, sizeof(m_dispBuf));
            if (m_ascCount % 2)   //Ϊ
            {
                memcpy(&m_dispBuf[0], pBuf, (maxDispLen - 1)); //ҪbufĴС, Ϊʾܴ,۷ͷ,ʾ,ԱbufĴС
            }
            else
            {
                memcpy(&m_dispBuf[0], pBuf, (maxDispLen));     //ҪbufĴС, Ϊʾܴ,۷ͷ,ʾ,ԱbufĴС
            }

            /*-------------------------------------------------------*/
            __mmi_displayEx(m_line++, 0, fontColor, backgroundColor, backgroundEnable, m_dispBuf);
            /*-------------------------------------------------------*/

            //ָ
            if (m_ascCount % 2)   //Ϊ
            {
                pBuf += (maxDispLen - 1);
                m_dispLen -= (maxDispLen - 1);     //ȼ, Ѿַ
            }
            else
            {
                pBuf += maxDispLen;
                m_dispLen -= maxDispLen;         //ȼ, Ѿַ
            }
            m_ascCount = 0;
            i = 0 - 1;
            continue;
        }
    }

    if (m_dispLen != 0)
    {
        memset(m_dispBuf, 0, sizeof(m_dispBuf));
        memcpy(&m_dispBuf[0], pBuf, m_dispLen);
        __mmi_displayEx(m_line++, 0, fontColor, backgroundColor, backgroundEnable, m_dispBuf);
    }
    return m_line;
}

/**
 * @brief ָĻϵָһıϢ(ֻ֧)
 * @param [in] line ʾ(ʹ 뷽ʽ ö)
 * @param [in] column ʾ(ʹ 뷽ʽ ö)
 * @param [in] fontColor ɫ
 * @param [in] backgroundColor ɫ
 * @param [in] backgroundEnable е(MMI_ENABLED-ɫʾ, MMI_DISABLED-ֻʾ屳ɫ)
 * @param [in] dispString ʾ
 * @retval һе
 */
int __mmi_displayEx(unsigned int line, unsigned int column, int fontColor, int backgroundColor, OFF_ON backgroundEnable, char *dispString)
{
    char szFixedMsg[50];
    char szFormat[50];
    char ucX = 0;
    const int m_linePix = LINEPIX;
    const int m_fontPix = FONTPIX;

    if (line != ALIG_TOP
        && line != ALIG_BOTTOM
        && line != ALIG_CENTER)
    {
        if((m_linePix * line) >= (s_mmi_lcd_param.srcSize.Height-m_fontPix))
        {
            return line;
        }
    }

    if (column != ALIG_LEFT
        && column != ALIG_RIGHT
        && column != ALIG_CENTER)
    {
        if (((m_fontPix/2)*column) >= (s_mmi_lcd_param.srcSize.Width-m_fontPix))
        {
            return line;
        }
    }

    //ʾ
    if (strlen(dispString) > SCREEN_WIDTH)
    {
        line = __mmi_display(line, fontColor, backgroundColor, backgroundEnable, dispString, SCREEN_WIDTH);
        return line;
    }

    memset(szFormat, 0, sizeof(szFormat));
    memset(szFixedMsg, 0, sizeof(szFixedMsg));

    //λ
    switch (column)
    {
        case ALIG_RIGHT:
            strcpy(szFixedMsg, dispString);
            ucX = SCREEN_WIDTH - strlen((char *)szFixedMsg);
            break;
        case ALIG_CENTER:
            strcpy(szFixedMsg, dispString);
            ucX = (SCREEN_WIDTH - strlen((char *)szFixedMsg)) / 2;
            break;
        default:
            strcpy(szFixedMsg, dispString);
            ucX = column == ALIG_LEFT ? 0: column;
            break;
    }

    //λ
    line = mmi_get_display_line(line);

    //
    mmi_set_font(1, 0,
                 (fontColor != MERCURY_NONE ? fontColor : s_mmi_lcd_param.fontColor),
                 (backgroundColor != MERCURY_NONE ? backgroundColor : s_mmi_lcd_param.backgroundColor));

    //ñɫ
    if (backgroundEnable == MMI_ENABLED)
    {
        mmi_set_background(line, backgroundColor);
    }

    //ʾ
    ScrGotoXY((m_fontPix/2)*ucX, ((LINEPIX - FONTPIX) / 2) + (m_linePix * line++));
    ScrPrintf(szFixedMsg);
    return line;
}

/**
 * @brief ָĻϵָһıϢ(ֻ֧)
 * @param [in] line ʾ(ʹ 뷽ʽ ö)
 * @param [in] column ʾ(ʹ 뷽ʽ ö)
 * @param [in] pszFmt ʾ
 * @retval һе
 */
int mmi_display(unsigned int line, unsigned int column, char *pszFmt, ...)
{
    char szMsg[255];
    va_list args;

    va_start(args, pszFmt);
#ifdef SDK_V1_8_8
    vsnprintf(szMsg, sizeof(szMsg), pszFmt, args);
#else
    vsprintf(szMsg, pszFmt, args);
#endif
    va_end(args);

    return __mmi_displayEx(line, column, MERCURY_NONE, MERCURY_NONE, MMI_DISABLED, szMsg);
}

/**
 * @brief ָĻϵָһıϢ(ֻ֧)
 * @param [in] line ʾ(ʹ 뷽ʽ ö)
 * @param [in] column ʾ(ʹ 뷽ʽ ö)
 * @param [in] fontColor ɫ
 * @param [in] backgroundColor ɫ
 * @param [in] backgroundEnable е(MMI_ENABLED-ɫʾ, MMI_DISABLED-ֻʾ屳ɫ)
 * @param [in] pszFmt ʾ
 * @retval һе
 */
int mmi_displayEx(unsigned int line, unsigned int column, int fontColor, int backgroundColor, OFF_ON backgroundEnable, char *pszFmt, ...)
{
    char szMsg[255];
    va_list args;

    va_start(args, pszFmt);
#ifdef SDK_V1_8_8
    vsnprintf(szMsg, sizeof(szMsg), pszFmt, args);
#else
    vsprintf(szMsg, pszFmt, args);
#endif
    va_end(args);

    return __mmi_displayEx(line, column, fontColor, backgroundColor, backgroundEnable, szMsg);
}

/**
 * @brief άչʾ
 * @param [in] line ʾ(ʹ 뷽ʽ ö)
 * @param [in] column ʾ(ʹ 뷽ʽ ö)
 * @param [in] width ά
 * @param [in] height ά߶
 * @param [in] marginWidth άױ߿
 * @param [in] paystr ʾ
 * @retval һе
 */
void mmi_display_qrcode(unsigned int line,
                        unsigned int column,
                        unsigned int width,
                        unsigned int height,
                        unsigned int marginWidth,
                        unsigned char *paystr)
{
    tsRect m_rect = { 0 };

    switch(line)
    {
        case ALIG_TOP:
            m_rect.Top = LINEPIX;
            line = 1;
            break;
        case ALIG_BOTTOM:
            m_rect.Top = s_mmi_lcd_param.srcSize.Height - height;
            break;
        case ALIG_CENTER:
            m_rect.Top = (s_mmi_lcd_param.srcSize.Height - height)/2;
            break;
        default:
            m_rect.Top = LINEPIX * line;
            break;
    }

    switch(column)
    {
        case ALIG_RIGHT:
            m_rect.Left = s_mmi_lcd_param.srcSize.Width - width;
            break;
        case ALIG_CENTER:
            m_rect.Left = (s_mmi_lcd_param.srcSize.Width - width)/2;
            break;
        default:
            m_rect.Left = (FONTPIX/2)*column;
            break;
    }
    m_rect.Width = width;
    m_rect.Height = height;

    ScrShowQrCodeExt(&m_rect, (char *)paystr, marginWidth);
}

/**
 * @brief ȡǰɫ
 */
int mmi_get_display_background_color(void)
{
    return s_mmi_lcd_param.backgroundColor;
}

/**
 * @brief ȡʾ
 */
int mmi_get_display_max_line(void)
{
    return (int)(s_mmi_lcd_param.srcSize.Height-ScrGetStatusBarHeight()) / LINEPIX;
}

/**
 * @brief ʾ
 * @param [in] line ʾ(ʹ 뷽ʽ ö)
 * @param [in] borderColor ߿ɫ
 * @param [in] bgColor ɫ
 * @param [in] percent ǰٷֱ
 */
void mmi_display_progress_bar(unsigned int line, int borderColor, int backgroundColor, unsigned int percent)
{
    tsRect rect = {0};

    if (line < 1)
        return;

    if (line != ALIG_TOP
        && line != ALIG_BOTTOM
        && line != ALIG_CENTER)
    {
        if((LINEPIX * line) >= (s_mmi_lcd_param.srcSize.Height-FONTPIX))
        {
            return;
        }
    }

    line = mmi_get_display_line(line);

    rect.Left = FONTPIX/2;
    rect.Top = LINEPIX * line;
    rect.Width = s_mmi_lcd_param.srcSize.Width-FONTPIX;
    rect.Height = FONTPIX;

    if (percent > 100)
        percent = 100;

    // Ȼ
    ScrDrawRect(&rect, 1, borderColor);

    // ٻƽ
    rect.Left += 1;
    rect.Top += 1;
    rect.Height -= 2;
    rect.Width = ((rect.Width-2)*percent)/100;
    ScrFillRect(&rect, backgroundColor);
}

static VOID __mmi_timer_dsiplay(ULONG param)
{
    if (s_mmi_timer_param.interrupt)
        goto DO_END;

    if (s_mmi_timer_param.flipEnable == MMI_ENABLED)
    {
        s_mmi_timer_param.time--;
        if (s_mmi_timer_param.time <= 0)
        {
            s_mmi_timer_param.interrupt = 1;
        }
    }
    else
    {
        s_mmi_timer_param.time++;
        if (s_mmi_timer_param.time >= s_mmi_timer_param.timeout)
        {
            s_mmi_timer_param.interrupt = 1;
        }
    }
    mmi_display(s_mmi_timer_param.line, s_mmi_timer_param.column, "%3d", s_mmi_timer_param.time);

DO_END:
    if (s_mmi_timer_param.interrupt)
    {
        DeactiveTimer(s_mmi_timer_param.timer);
        s_mmi_timer_param.interrupt = 0;
    }
}

/**
 * @brief ʱʾ
 * @param [in] line ʾ(ʹ 뷽ʽ ö)
 * @param [in] column ʾ(ʹ 뷽ʽ ö)
 * @param [in] flipEnable õʱ (MMI_DISABLED-ʱʾ, MMI_ENABLED-ʱʾ)
 * @param [in] timeout ʱʱ
 */
void mmi_dsiplay_timer_start(unsigned int line, unsigned int column, OFF_ON flipEnable, int timeout)
{
    TIMER_CONFIG_S m_timer_cfg;

    s_mmi_timer_param.interrupt = 0;
    s_mmi_timer_param.line = line;
    s_mmi_timer_param.column = column;

    s_mmi_timer_param.flipEnable = flipEnable;
    s_mmi_timer_param.timeout = timeout;
    s_mmi_timer_param.time = 0;
    if (s_mmi_timer_param.flipEnable == MMI_ENABLED)
        s_mmi_timer_param.time = timeout;

    if (s_mmi_timer_param.timer == NULL)
    {
        memset(&m_timer_cfg, 0, sizeof(TIMER_CONFIG_S));
        m_timer_cfg.timer_name  = "disptimer";
        m_timer_cfg.timer_fun   = __mmi_timer_dsiplay;

        m_timer_cfg.input           = 0;
        m_timer_cfg.timer_expire    = 1000;
        m_timer_cfg.auto_activate   = TIMER_AUTO_ACTIVATE;
        m_timer_cfg.period          = TIMER_PERIOD;
        s_mmi_timer_param.timer     = CreateTimer(&m_timer_cfg);
    }
    else
    {
        ChangeTimer(s_mmi_timer_param.timer, __mmi_timer_dsiplay, 1000);
        ActiveTimer(s_mmi_timer_param.timer);
    }
}

/**
 * @brief ʱֹͣʱ
 */
void mmi_dsiplay_timer_stop(void)
{
    s_mmi_timer_param.interrupt = 1;
}

